@@ -12,10 +12,9 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.  */
+   along with this program; if not, see <http://gnu.org/licenses/gpl.html>
 
-/* When this file is included, it may be preceded only by preprocessor
+   When this file is included, it may be preceded only by preprocessor
    declarations.  Thanks to AIX.  Therefore we include it right after
    "config.h", not later.  */
 
